@@ -8,16 +8,29 @@ import collections.abc as cabc
 
 from xonsh.ast import CtxAwareTransformer
 from xonsh.parser import Parser
-from xonsh.tools import (subproc_toks, find_next_break, get_logical_line,
-                         replace_logical_line, balanced_parens)
-from xonsh.built_ins import load_builtins, unload_builtins
+from xonsh.tools import (
+    subproc_toks,
+    find_next_break,
+    get_logical_line,
+    replace_logical_line,
+    balanced_parens,
+)
+from xonsh.built_ins import load_builtins, unload_builtins, load_proxies, unload_proxies
 
 
 class Execer(object):
     """Executes xonsh code in a context."""
 
-    def __init__(self, filename='<xonsh-code>', debug_level=0, parser_args=None,
-                 unload=True, xonsh_ctx=None, scriptcache=True, cacheall=False):
+    def __init__(
+        self,
+        filename="<xonsh-code>",
+        debug_level=0,
+        parser_args=None,
+        unload=True,
+        xonsh_ctx=None,
+        scriptcache=True,
+        cacheall=False,
+    ):
         """Parameters
         ----------
         filename : str, optional
@@ -29,7 +42,7 @@ class Execer(object):
         unload : bool, optional
             Whether or not to unload xonsh builtins upon deletion.
         xonsh_ctx : dict or None, optional
-            Xonsh xontext to load as builtins.__xonsh_ctx__
+            Xonsh xontext to load as builtins.__xonsh__.ctx
         scriptcache : bool, optional
             Whether or not to use a precompiled bytecode cache when execing
             code, default: True.
@@ -46,12 +59,14 @@ class Execer(object):
         self.cacheall = cacheall
         self.ctxtransformer = CtxAwareTransformer(self.parser)
         load_builtins(execer=self, ctx=xonsh_ctx)
+        load_proxies()
 
     def __del__(self):
         if self.unload:
+            unload_proxies()
             unload_builtins()
 
-    def parse(self, input, ctx, mode='exec', filename=None, transform=True):
+    def parse(self, input, ctx, mode="exec", filename=None, transform=True):
         """Parses xonsh code in a context-aware fashion. For context-free
         parsing, please use the Parser class directly or pass in
         transform=False.
@@ -59,8 +74,9 @@ class Execer(object):
         if filename is None:
             filename = self.filename
         if not transform:
-            return self.parser.parse(input, filename=filename, mode=mode,
-                                     debug_level=(self.debug_level > 2))
+            return self.parser.parse(
+                input, filename=filename, mode=mode, debug_level=(self.debug_level > 2)
+            )
 
         # Parsing actually happens in a couple of phases. The first is a
         # shortcut for a context-free parser. Normally, all subprocess
@@ -90,12 +106,21 @@ class Execer(object):
             ctx = set()
         elif isinstance(ctx, cabc.Mapping):
             ctx = set(ctx.keys())
-        tree = self.ctxtransformer.ctxvisit(tree, input, ctx, mode=mode,
-                                            debug_level=self.debug_level)
+        tree = self.ctxtransformer.ctxvisit(
+            tree, input, ctx, mode=mode, debug_level=self.debug_level
+        )
         return tree
 
-    def compile(self, input, mode='exec', glbs=None, locs=None, stacklevel=2,
-                filename=None, transform=True):
+    def compile(
+        self,
+        input,
+        mode="exec",
+        glbs=None,
+        locs=None,
+        stacklevel=2,
+        filename=None,
+        transform=True,
+    ):
         """Compiles xonsh code into a Python code object, which may then
         be execed or evaled.
         """
@@ -106,63 +131,76 @@ class Execer(object):
             glbs = frame.f_globals if glbs is None else glbs
             locs = frame.f_locals if locs is None else locs
         ctx = set(dir(builtins)) | set(glbs.keys()) | set(locs.keys())
-        tree = self.parse(input, ctx, mode=mode, filename=filename,
-                          transform=transform)
+        tree = self.parse(input, ctx, mode=mode, filename=filename, transform=transform)
         if tree is None:
             return None  # handles comment only input
         code = compile(tree, filename, mode)
         return code
 
-    def eval(self, input, glbs=None, locs=None, stacklevel=2,
-             filename=None, transform=True):
+    def eval(
+        self, input, glbs=None, locs=None, stacklevel=2, filename=None, transform=True
+    ):
         """Evaluates (and returns) xonsh code."""
         if isinstance(input, types.CodeType):
             code = input
         else:
             if filename is None:
                 filename = self.filename
-            code = self.compile(input=input,
-                                glbs=glbs,
-                                locs=locs,
-                                mode='eval',
-                                stacklevel=stacklevel,
-                                filename=filename,
-                                transform=transform)
+            code = self.compile(
+                input=input,
+                glbs=glbs,
+                locs=locs,
+                mode="eval",
+                stacklevel=stacklevel,
+                filename=filename,
+                transform=transform,
+            )
         if code is None:
             return None  # handles comment only input
         return eval(code, glbs, locs)
 
-    def exec(self, input, mode='exec', glbs=None, locs=None, stacklevel=2,
-             filename=None, transform=True):
+    def exec(
+        self,
+        input,
+        mode="exec",
+        glbs=None,
+        locs=None,
+        stacklevel=2,
+        filename=None,
+        transform=True,
+    ):
         """Execute xonsh code."""
         if isinstance(input, types.CodeType):
             code = input
         else:
             if filename is None:
                 filename = self.filename
-            code = self.compile(input=input,
-                                glbs=glbs,
-                                locs=locs,
-                                mode=mode,
-                                stacklevel=stacklevel,
-                                filename=filename,
-                                transform=transform)
+            code = self.compile(
+                input=input,
+                glbs=glbs,
+                locs=locs,
+                mode=mode,
+                stacklevel=stacklevel,
+                filename=filename,
+                transform=transform,
+            )
         if code is None:
             return None  # handles comment only input
         return exec(code, glbs, locs)
 
-    def _print_debug_wrapping(self, line, sbpline, last_error_line, last_error_col,
-                              maxcol=None):
+    def _print_debug_wrapping(
+        self, line, sbpline, last_error_line, last_error_col, maxcol=None
+    ):
         """print some debugging info if asked for."""
         if self.debug_level > 1:
-            msg = ('{0}:{1}:{2}{3} - {4}\n'
-                   '{0}:{1}:{2}{3} + {5}')
-            mstr = '' if maxcol is None else ':' + str(maxcol)
-            msg = msg.format(self.filename, last_error_line,
-                             last_error_col, mstr, line, sbpline)
+            msg = "{0}:{1}:{2}{3} - {4}\n" "{0}:{1}:{2}{3} + {5}"
+            mstr = "" if maxcol is None else ":" + str(maxcol)
+            msg = msg.format(
+                self.filename, last_error_line, last_error_col, mstr, line, sbpline
+            )
             print(msg, file=sys.stderr)
 
-    def _parse_ctx_free(self, input, mode='exec', filename=None, logical_input=False):
+    def _parse_ctx_free(self, input, mode="exec", filename=None, logical_input=False):
         last_error_line = last_error_col = -1
         parsed = False
         original_error = None
@@ -171,10 +209,12 @@ class Execer(object):
             filename = self.filename
         while not parsed:
             try:
-                tree = self.parser.parse(input,
-                                         filename=filename,
-                                         mode=mode,
-                                         debug_level=(self.debug_level > 2))
+                tree = self.parser.parse(
+                    input,
+                    filename=filename,
+                    mode=mode,
+                    debug_level=(self.debug_level > 2),
+                )
                 parsed = True
             except IndentationError as e:
                 if original_error is None:
@@ -184,9 +224,10 @@ class Execer(object):
             except SyntaxError as e:
                 if original_error is None:
                     original_error = e
-                if (e.loc is None) or (last_error_line == e.loc.lineno and
-                                       last_error_col in (e.loc.column + 1,
-                                                          e.loc.column)):
+                if (e.loc is None) or (
+                    last_error_line == e.loc.lineno
+                    and last_error_col in (e.loc.column + 1, e.loc.column)
+                ):
                     raise original_error from None
                 elif last_error_line != e.loc.lineno:
                     original_error = e
@@ -196,17 +237,18 @@ class Execer(object):
                 lines = input.splitlines()
                 line, nlogical, idx = get_logical_line(lines, idx)
                 if nlogical > 1 and not logical_input:
-                    _, sbpline = self._parse_ctx_free(line, mode=mode,
-                                                      filename=filename,
-                                                      logical_input=True)
-                    self._print_debug_wrapping(line, sbpline, last_error_line,
-                                               last_error_col, maxcol=None)
+                    _, sbpline = self._parse_ctx_free(
+                        line, mode=mode, filename=filename, logical_input=True
+                    )
+                    self._print_debug_wrapping(
+                        line, sbpline, last_error_line, last_error_col, maxcol=None
+                    )
                     replace_logical_line(lines, sbpline, idx, nlogical)
                     last_error_col += 3
-                    input = '\n'.join(lines)
+                    input = "\n".join(lines)
                     continue
-                if input.endswith('\n'):
-                    lines.append('')
+                if input.endswith("\n"):
+                    lines.append("")
                 if len(line.strip()) == 0:
                     # whitespace only lines are not valid syntax in Python's
                     # interactive mode='single', who knew?! Just ignore them.
@@ -214,19 +256,21 @@ class Execer(object):
                     # numbers reported, but should only affect interactive mode
                     del lines[idx]
                     last_error_line = last_error_col = -1
-                    input = '\n'.join(lines)
+                    input = "\n".join(lines)
                     continue
 
-                if last_error_line > 1 and lines[idx - 1].rstrip()[-1:] == ':':
+                if last_error_line > 1 and lines[idx - 1].rstrip()[-1:] == ":":
                     # catch non-indented blocks and raise error.
                     prev_indent = len(lines[idx - 1]) - len(lines[idx - 1].lstrip())
                     curr_indent = len(lines[idx]) - len(lines[idx].lstrip())
                     if prev_indent == curr_indent:
                         raise original_error
                 lexer = self.parser.lexer
-                maxcol = None if greedy else find_next_break(line,
-                                                             mincol=last_error_col,
-                                                             lexer=lexer)
+                maxcol = (
+                    None
+                    if greedy
+                    else find_next_break(line, mincol=last_error_col, lexer=lexer)
+                )
                 if not greedy and maxcol in (e.loc.column + 1, e.loc.column):
                     # go greedy the first time if the syntax error was because
                     # we hit an end token out of place. This usually indicates
@@ -234,15 +278,16 @@ class Execer(object):
                     if not balanced_parens(line, maxcol=maxcol):
                         greedy = True
                         maxcol = None
-                sbpline = subproc_toks(line, returnline=True, greedy=greedy,
-                                       maxcol=maxcol, lexer=lexer)
+                sbpline = subproc_toks(
+                    line, returnline=True, greedy=greedy, maxcol=maxcol, lexer=lexer
+                )
                 if sbpline is None:
                     # subprocess line had no valid tokens,
-                    if len(line.partition('#')[0].strip()) == 0:
+                    if len(line.partition("#")[0].strip()) == 0:
                         # likely because it only contained a comment.
                         del lines[idx]
                         last_error_line = last_error_col = -1
-                        input = '\n'.join(lines)
+                        input = "\n".join(lines)
                         continue
                     elif not greedy:
                         greedy = True
@@ -250,8 +295,9 @@ class Execer(object):
                     else:
                         # or for some other syntax error
                         raise original_error
-                elif sbpline[last_error_col:].startswith('![![') or \
-                        sbpline.lstrip().startswith('![!['):
+                elif sbpline[last_error_col:].startswith(
+                    "![!["
+                ) or sbpline.lstrip().startswith("![!["):
                     # if we have already wrapped this in subproc tokens
                     # and it still doesn't work, adding more won't help
                     # anything
@@ -261,9 +307,10 @@ class Execer(object):
                     else:
                         raise original_error
                 # replace the line
-                self._print_debug_wrapping(line, sbpline, last_error_line,
-                                           last_error_col, maxcol=maxcol)
+                self._print_debug_wrapping(
+                    line, sbpline, last_error_line, last_error_col, maxcol=maxcol
+                )
                 replace_logical_line(lines, sbpline, idx, nlogical)
                 last_error_col += 3
-                input = '\n'.join(lines)
+                input = "\n".join(lines)
         return tree, input
