@@ -36,6 +36,7 @@ import threading
 import traceback
 import warnings
 import operator
+import ast
 
 # adding imports from further xonsh modules is discouraged to avoid circular
 # dependencies
@@ -96,7 +97,8 @@ class XonshCalledProcessError(XonshError, subprocess.CalledProcessError):
 def expand_path(s, expand_user=True):
     """Takes a string path and expands ~ to home if expand_user is set
     and environment vars if EXPAND_ENV_VARS is set."""
-    env = getattr(builtins, "__xonsh_env__", os_environ)
+    session = getattr(builtins, "__xonsh__", None)
+    env = os_environ if session is None else getattr(session, "env", os_environ)
     if env.get("EXPAND_ENV_VARS", False):
         s = expandvars(s)
     if expand_user:
@@ -117,7 +119,8 @@ def _expandpath(path):
     """Performs environment variable / user expansion on a given path
     if EXPAND_ENV_VARS is set.
     """
-    env = getattr(builtins, "__xonsh_env__", os_environ)
+    session = getattr(builtins, "__xonsh__", None)
+    env = os_environ if session is None else getattr(session, "env", os_environ)
     expand_user = env.get("EXPAND_ENV_VARS", False)
     return expand_path(path, expand_user=expand_user)
 
@@ -126,7 +129,8 @@ def decode_bytes(b):
     """Tries to decode the bytes using XONSH_ENCODING if available,
     otherwise using sys.getdefaultencoding().
     """
-    env = getattr(builtins, "__xonsh_env__", os_environ)
+    session = getattr(builtins, "__xonsh__", None)
+    env = os_environ if session is None else getattr(session, "env", os_environ)
     enc = env.get("XONSH_ENCODING") or DEFAULT_ENCODING
     err = env.get("XONSH_ENCODING_ERRORS") or "strict"
     return b.decode(encoding=enc, errors=err)
@@ -146,7 +150,7 @@ def findfirst(s, substrs):
     return i, result
 
 
-class EnvPath(collections.MutableSequence):
+class EnvPath(cabc.MutableSequence):
     """A class that implements an environment path, which is a list of
     strings. Provides a custom method that expands all paths if the
     relevant env variable has been set.
@@ -164,7 +168,7 @@ class EnvPath(collections.MutableSequence):
                 # decode bytes to a string and then split based on
                 # the default path separator
                 self._l = decode_bytes(args).split(os.pathsep)
-            elif isinstance(args, collections.Iterable):
+            elif isinstance(args, cabc.Iterable):
                 # put everything in a list -before- performing the type check
                 # in order to be able to retrieve it later, for cases such as
                 # when a generator expression was passed as an argument
@@ -290,7 +294,7 @@ END_TOK_TYPES = LazyObject(
     lambda: frozenset(["SEMI", "AND", "OR", "RPAREN"]), globals(), "END_TOK_TYPES"
 )
 RE_END_TOKS = LazyObject(
-    lambda: re.compile("(;|and|\&\&|or|\|\||\))"), globals(), "RE_END_TOKS"
+    lambda: re.compile(r"(;|and|\&\&|or|\|\||\))"), globals(), "RE_END_TOKS"
 )
 LPARENS = LazyObject(
     lambda: frozenset(
@@ -313,7 +317,7 @@ def balanced_parens(line, mincol=0, maxcol=None, lexer=None):
     """Determines if parentheses are balanced in an expression."""
     line = line[mincol:maxcol]
     if lexer is None:
-        lexer = builtins.__xonsh_execer__.parser.lexer
+        lexer = builtins.__xonsh__.execer.parser.lexer
     if "(" not in line and ")" not in line:
         return True
     cnt = 0
@@ -336,7 +340,7 @@ def find_next_break(line, mincol=0, lexer=None):
     if mincol >= 1:
         line = line[mincol:]
     if lexer is None:
-        lexer = builtins.__xonsh_execer__.parser.lexer
+        lexer = builtins.__xonsh__.execer.parser.lexer
     if RE_END_TOKS.search(line) is None:
         return None
     maxcol = None
@@ -374,7 +378,7 @@ def subproc_toks(
     normal parentheses. Greedy is False by default.
     """
     if lexer is None:
-        lexer = builtins.__xonsh_execer__.parser.lexer
+        lexer = builtins.__xonsh__.execer.parser.lexer
     if maxcol is None:
         maxcol = len(line) + 1
     lexer.reset()
@@ -511,8 +515,8 @@ def get_line_continuation():
     """
     if (
         ON_WINDOWS
-        and hasattr(builtins, "__xonsh_env__")
-        and builtins.__xonsh_env__.get("XONSH_INTERACTIVE", False)
+        and hasattr(builtins.__xonsh__, "env")
+        and builtins.__xonsh__.env.get("XONSH_INTERACTIVE", False)
     ):
         return " \\"
     else:
@@ -674,7 +678,7 @@ def get_sep():
     """ Returns the appropriate filepath separator char depending on OS and
     xonsh options set
     """
-    if ON_WINDOWS and builtins.__xonsh_env__.get("FORCE_POSIX_PATHS"):
+    if ON_WINDOWS and builtins.__xonsh__.env.get("FORCE_POSIX_PATHS"):
         return os.altsep
     else:
         return os.sep
@@ -770,7 +774,7 @@ def _executables_in_posix(path):
 def _executables_in_windows(path):
     if not os.path.isdir(path):
         return
-    extensions = builtins.__xonsh_env__["PATHEXT"]
+    extensions = builtins.__xonsh__.env["PATHEXT"]
     if PYTHON_VERSION_INFO < (3, 5, 0):
         for fname in os.listdir(path):
             fpath = os.path.join(path, fname)
@@ -867,7 +871,7 @@ def suggest_commands(cmd, env, aliases):
 
 def print_exception(msg=None):
     """Print exceptions with/without traceback."""
-    env = getattr(builtins, "__xonsh_env__", None)
+    env = getattr(builtins.__xonsh__, "env", None)
     # flags indicating whether the traceback options have been manually set
     if env is None:
         env = os_environ
@@ -1469,6 +1473,45 @@ def to_completions_display_value(x):
     return x
 
 
+def is_str_str_dict(x):
+    """Tests if something is a str:str dictionary"""
+    return isinstance(x, dict) and all(
+        isinstance(k, str) and isinstance(v, str) for k, v in x.items()
+    )
+
+
+def to_dict(x):
+    """Converts a string to a dictionary"""
+    if isinstance(x, dict):
+        return x
+    try:
+        x = ast.literal_eval(x)
+    except (ValueError, SyntaxError):
+        msg = '"{}" can not be converted to Python dictionary.'.format(x)
+        warnings.warn(msg, RuntimeWarning)
+        x = dict()
+    return x
+
+
+def to_str_str_dict(x):
+    """Converts a string to str:str dictionary"""
+    if is_str_str_dict(x):
+        return x
+    x = to_dict(x)
+    if not is_str_str_dict(x):
+        msg = '"{}" can not be converted to str:str dictionary.'.format(x)
+        warnings.warn(msg, RuntimeWarning)
+        x = dict()
+    return x
+
+
+def dict_to_str(x):
+    """Converts a dictionary to a string"""
+    if not x or len(x) == 0:
+        return ""
+    return str(x)
+
+
 def setup_win_unicode_console(enable):
     """"Enables or disables unicode display on windows."""
     try:
@@ -1610,7 +1653,7 @@ def dynamic_cwd_tuple_to_str(x):
 
 
 RE_HISTORY_TUPLE = LazyObject(
-    lambda: re.compile("([-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?)\s*([A-Za-z]*)"),
+    lambda: re.compile(r"([-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?)\s*([A-Za-z]*)"),
     globals(),
     "RE_HISTORY_TUPLE",
 )
@@ -1640,7 +1683,7 @@ def format_color(string, **kwargs):
     shell instances method of the same name. The results of this function should
     be directly usable by print_color().
     """
-    return builtins.__xonsh_shell__.shell.format_color(string, **kwargs)
+    return builtins.__xonsh__.shell.shell.format_color(string, **kwargs)
 
 
 def print_color(string, **kwargs):
@@ -1648,24 +1691,24 @@ def print_color(string, **kwargs):
     method of the same name. Colors will be formatted if they have not already
     been.
     """
-    builtins.__xonsh_shell__.shell.print_color(string, **kwargs)
+    builtins.__xonsh__.shell.shell.print_color(string, **kwargs)
 
 
 def color_style_names():
     """Returns an iterable of all available style names."""
-    return builtins.__xonsh_shell__.shell.color_style_names()
+    return builtins.__xonsh__.shell.shell.color_style_names()
 
 
 def color_style():
     """Returns the current color map."""
-    return builtins.__xonsh_shell__.shell.color_style()
+    return builtins.__xonsh__.shell.shell.color_style()
 
 
 def _token_attr_from_stylemap(stylemap):
     """yields tokens attr, and index from a stylemap """
     import prompt_toolkit as ptk
 
-    if builtins.__xonsh_shell__.shell_type == "prompt_toolkit1":
+    if builtins.__xonsh__.shell.shell_type == "prompt_toolkit1":
         style = ptk.styles.style_from_dict(stylemap)
         for token in stylemap:
             yield token, style.token_to_attrs[token]
@@ -1680,7 +1723,7 @@ def _token_attr_from_stylemap(stylemap):
 
 def _get_color_lookup_table():
     """Returns the prompt_toolkit win32 ColorLookupTable """
-    if builtins.__xonsh_shell__.shell_type == "prompt_toolkit1":
+    if builtins.__xonsh__.shell.shell_type == "prompt_toolkit1":
         from prompt_toolkit.terminal.win32_output import ColorLookupTable
     else:
         from prompt_toolkit.output.win32 import ColorLookupTable
@@ -1785,8 +1828,8 @@ def hardcode_colors_for_win10(style_map):
        in conhost.exe
     """
     modified_style = {}
-    if not builtins.__xonsh_env__["PROMPT_TOOLKIT_COLOR_DEPTH"]:
-        builtins.__xonsh_env__["PROMPT_TOOLKIT_COLOR_DEPTH"] = "DEPTH_24_BIT"
+    if not builtins.__xonsh__.env["PROMPT_TOOLKIT_COLOR_DEPTH"]:
+        builtins.__xonsh__.env["PROMPT_TOOLKIT_COLOR_DEPTH"] = "DEPTH_24_BIT"
     # Replace all ansi colors with hardcoded colors to avoid unreadable defaults
     # in conhost.exe
     for token, style_str in style_map.items():
@@ -1832,7 +1875,7 @@ def intensify_colors_for_cmd_exe(style_map):
         6: "ansibrightyellow",  # subst yellow with bright yellow
         9: "ansicyan",  # subst intense blue with dark cyan (more readable)
     }
-    if builtins.__xonsh_shell__.shell_type == "prompt_toolkit1":
+    if builtins.__xonsh__.shell.shell_type == "prompt_toolkit1":
         replace_colors = ansicolors_to_ptk1_names(replace_colors)
     for token, idx, _ in _get_color_indexes(style_map):
         if idx in replace_colors:
@@ -1845,10 +1888,9 @@ def intensify_colors_on_win_setter(enable):
     environment variable.
     """
     enable = to_bool(enable)
-    if hasattr(builtins, "__xonsh_shell__"):
-        builtins.__xonsh_shell__.shell.styler.trap.clear()
-        if hasattr(builtins.__xonsh_shell__.shell.styler, "style_name"):
-            delattr(builtins.__xonsh_shell__.shell.styler, "style_name")
+    if hasattr(builtins.__xonsh__, "shell"):
+        if hasattr(builtins.__xonsh__.shell.shell.styler, "style_name"):
+            delattr(builtins.__xonsh__.shell.shell.styler, "style_name")
     return enable
 
 
@@ -1858,8 +1900,8 @@ def format_std_prepost(template, env=None):
     """
     if not template:
         return ""
-    env = builtins.__xonsh_env__ if env is None else env
-    shell = builtins.__xonsh_shell__.shell
+    env = builtins.__xonsh__.env if env is None else env
+    shell = builtins.__xonsh__.shell.shell
     try:
         s = shell.prompt_formatter(template)
     except Exception:
@@ -2003,7 +2045,7 @@ def POSIX_ENVVAR_REGEX():
 def expandvars(path):
     """Expand shell variables of the forms $var, ${var} and %var%.
     Unknown variables are left unchanged."""
-    env = builtins.__xonsh_env__
+    env = builtins.__xonsh__.env
     if isinstance(path, bytes):
         path = path.decode(
             encoding=env.get("XONSH_ENCODING"), errors=env.get("XONSH_ENCODING_ERRORS")
@@ -2118,11 +2160,11 @@ def _dotglobstr(s):
 
 
 def _iglobpath(s, ignore_case=False, sort_result=None, include_dotfiles=None):
-    s = builtins.__xonsh_expand_path__(s)
+    s = builtins.__xonsh__.expand_path(s)
     if sort_result is None:
-        sort_result = builtins.__xonsh_env__.get("GLOB_SORTED")
+        sort_result = builtins.__xonsh__.env.get("GLOB_SORTED")
     if include_dotfiles is None:
-        include_dotfiles = builtins.__xonsh_env__.get("DOTGLOB")
+        include_dotfiles = builtins.__xonsh__.env.get("DOTGLOB")
     if ignore_case:
         s = expand_case_matching(s)
     if sys.version_info > (3, 5):
@@ -2180,7 +2222,7 @@ def ensure_timestamp(t, datetime_format=None):
     except (ValueError, TypeError):
         pass
     if datetime_format is None:
-        datetime_format = builtins.__xonsh_env__["XONSH_DATETIME_FORMAT"]
+        datetime_format = builtins.__xonsh__.env["XONSH_DATETIME_FORMAT"]
     if isinstance(t, datetime.datetime):
         t = t.timestamp()
     else:
@@ -2190,7 +2232,7 @@ def ensure_timestamp(t, datetime_format=None):
 
 def format_datetime(dt):
     """Format datetime object to string base on $XONSH_DATETIME_FORMAT Env."""
-    format_ = builtins.__xonsh_env__["XONSH_DATETIME_FORMAT"]
+    format_ = builtins.__xonsh__.env["XONSH_DATETIME_FORMAT"]
     return dt.strftime(format_)
 
 
